@@ -1,10 +1,16 @@
-import unittest
-from django.contrib.gis.gdal import OGRGeometry, OGRGeomType, \
-    OGRException, OGRIndexError, SpatialReference, CoordTransform, \
-    gdal_version
-from django.contrib.gis.tests.geometries import *
+from binascii import b2a_hex
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
-class OGRGeomTest(unittest.TestCase):
+from django.contrib.gis.gdal import (OGRGeometry, OGRGeomType, OGRException,
+    OGRIndexError, SpatialReference, CoordTransform, GDAL_VERSION)
+from django.contrib.gis.gdal.prototypes.geom import GEOJSON
+from django.contrib.gis.geometry.test_data import TestDataMixin
+from django.utils import unittest
+
+class OGRGeomTest(unittest.TestCase, TestDataMixin):
     "This tests the OGR Geometry."
 
     def test00a_geomtype(self):
@@ -46,21 +52,44 @@ class OGRGeomTest(unittest.TestCase):
         self.assertEqual(0, gt.num)
         self.assertEqual('Unknown', gt.name)
 
+    def test00b_geomtype_25d(self):
+        "Testing OGRGeomType object with 25D types."
+        wkb25bit = OGRGeomType.wkb25bit
+        self.assertTrue(OGRGeomType(wkb25bit + 1) == 'Point25D')
+        self.assertTrue(OGRGeomType('MultiLineString25D') == (5 + wkb25bit))
+        self.assertEqual('GeometryCollectionField', OGRGeomType('GeometryCollection25D').django)
+
     def test01a_wkt(self):
         "Testing WKT output."
-        for g in wkt_out:
+        for g in self.geometries.wkt_out:
             geom = OGRGeometry(g.wkt)
             self.assertEqual(g.wkt, geom.wkt)
 
+    def test01a_ewkt(self):
+        "Testing EWKT input/output."
+        for ewkt_val in ('POINT (1 2 3)', 'LINEARRING (0 0,1 1,2 1,0 0)'):
+            # First with ewkt output when no SRID in EWKT
+            self.assertEqual(ewkt_val, OGRGeometry(ewkt_val).ewkt)
+            # No test consumption with an SRID specified.
+            ewkt_val = 'SRID=4326;%s' % ewkt_val
+            geom = OGRGeometry(ewkt_val)
+            self.assertEqual(ewkt_val, geom.ewkt)
+            self.assertEqual(4326, geom.srs.srid)
+
     def test01b_gml(self):
         "Testing GML output."
-        for g in wkt_out:
+        for g in self.geometries.wkt_out:
             geom = OGRGeometry(g.wkt)
-            self.assertEqual(g.gml, geom.gml)
+            exp_gml = g.gml
+            if GDAL_VERSION >= (1, 8):
+                # In GDAL 1.8, the non-conformant GML tag  <gml:GeometryCollection> was
+                # replaced with <gml:MultiGeometry>.
+                exp_gml = exp_gml.replace('GeometryCollection', 'MultiGeometry')
+            self.assertEqual(exp_gml, geom.gml)
 
     def test01c_hex(self):
         "Testing HEX input/output."
-        for g in hex_wkt:
+        for g in self.geometries.hex_wkt:
             geom1 = OGRGeometry(g.wkt)
             self.assertEqual(g.hex, geom1.hex)
             # Constructing w/HEX
@@ -69,8 +98,7 @@ class OGRGeomTest(unittest.TestCase):
 
     def test01d_wkb(self):
         "Testing WKB input/output."
-        from binascii import b2a_hex
-        for g in hex_wkt:
+        for g in self.geometries.hex_wkt:
             geom1 = OGRGeometry(g.wkt)
             wkb = geom1.wkb
             self.assertEqual(b2a_hex(wkb).upper(), g.hex)
@@ -80,9 +108,8 @@ class OGRGeomTest(unittest.TestCase):
 
     def test01e_json(self):
         "Testing GeoJSON input/output."
-        from django.contrib.gis.gdal.prototypes.geom import GEOJSON
         if not GEOJSON: return
-        for g in json_geoms:
+        for g in self.geometries.json_geoms:
             geom = OGRGeometry(g.wkt)
             if not hasattr(g, 'not_equal'):
                 self.assertEqual(g.json, geom.json)
@@ -93,7 +120,7 @@ class OGRGeomTest(unittest.TestCase):
         "Testing Point objects."
 
         prev = OGRGeometry('POINT(0 0)')
-        for p in points:
+        for p in self.geometries.points:
             if not hasattr(p, 'z'): # No 3D
                 pnt = OGRGeometry(p.wkt)
                 self.assertEqual(1, pnt.geom_type)
@@ -104,8 +131,7 @@ class OGRGeomTest(unittest.TestCase):
 
     def test03_multipoints(self):
         "Testing MultiPoint objects."
-
-        for mp in multipoints:
+        for mp in self.geometries.multipoints:
             mgeom1 = OGRGeometry(mp.wkt) # First one from WKT
             self.assertEqual(4, mgeom1.geom_type)
             self.assertEqual('MULTIPOINT', mgeom1.geom_name)
@@ -116,50 +142,50 @@ class OGRGeomTest(unittest.TestCase):
                 mgeom3.add(g.wkt) # should take WKT as well
             self.assertEqual(mgeom1, mgeom2) # they should equal
             self.assertEqual(mgeom1, mgeom3)
-            self.assertEqual(mp.points, mgeom2.tuple)
+            self.assertEqual(mp.coords, mgeom2.coords)
             self.assertEqual(mp.n_p, mgeom2.point_count)
-                                                                            
+
     def test04_linestring(self):
         "Testing LineString objects."
         prev = OGRGeometry('POINT(0 0)')
-        for ls in linestrings:
+        for ls in self.geometries.linestrings:
             linestr = OGRGeometry(ls.wkt)
             self.assertEqual(2, linestr.geom_type)
             self.assertEqual('LINESTRING', linestr.geom_name)
             self.assertEqual(ls.n_p, linestr.point_count)
-            self.assertEqual(ls.tup, linestr.tuple)
+            self.assertEqual(ls.coords, linestr.tuple)
             self.assertEqual(True, linestr == OGRGeometry(ls.wkt))
             self.assertEqual(True, linestr != prev)
             self.assertRaises(OGRIndexError, linestr.__getitem__, len(linestr))
             prev = linestr
 
             # Testing the x, y properties.
-            x = [tmpx for tmpx, tmpy in ls.tup]
-            y = [tmpy for tmpx, tmpy in ls.tup]
+            x = [tmpx for tmpx, tmpy in ls.coords]
+            y = [tmpy for tmpx, tmpy in ls.coords]
             self.assertEqual(x, linestr.x)
             self.assertEqual(y, linestr.y)
 
     def test05_multilinestring(self):
         "Testing MultiLineString objects."
         prev = OGRGeometry('POINT(0 0)')
-        for mls in multilinestrings:
+        for mls in self.geometries.multilinestrings:
             mlinestr = OGRGeometry(mls.wkt)
             self.assertEqual(5, mlinestr.geom_type)
             self.assertEqual('MULTILINESTRING', mlinestr.geom_name)
             self.assertEqual(mls.n_p, mlinestr.point_count)
-            self.assertEqual(mls.tup, mlinestr.tuple)
+            self.assertEqual(mls.coords, mlinestr.tuple)
             self.assertEqual(True, mlinestr == OGRGeometry(mls.wkt))
             self.assertEqual(True, mlinestr != prev)
             prev = mlinestr
             for ls in mlinestr:
                 self.assertEqual(2, ls.geom_type)
                 self.assertEqual('LINESTRING', ls.geom_name)
-            self.assertRaises(OGRIndexError, mlinestr.__getitem__, len(mlinestr)) 
+            self.assertRaises(OGRIndexError, mlinestr.__getitem__, len(mlinestr))
 
     def test06_linearring(self):
         "Testing LinearRing objects."
         prev = OGRGeometry('POINT(0 0)')
-        for rr in linearrings:
+        for rr in self.geometries.linearrings:
             lr = OGRGeometry(rr.wkt)
             #self.assertEqual(101, lr.geom_type.num)
             self.assertEqual('LINEARRING', lr.geom_name)
@@ -172,12 +198,12 @@ class OGRGeomTest(unittest.TestCase):
         "Testing Polygon objects."
 
         # Testing `from_bbox` class method
-        bbox =  (-180,-90,180,90) 
+        bbox =  (-180,-90,180,90)
         p = OGRGeometry.from_bbox( bbox )
         self.assertEqual(bbox, p.extent)
 
         prev = OGRGeometry('POINT(0 0)')
-        for p in polygons:
+        for p in self.geometries.polygons:
             poly = OGRGeometry(p.wkt)
             self.assertEqual(3, poly.geom_type)
             self.assertEqual('POLYGON', poly.geom_name)
@@ -193,13 +219,13 @@ class OGRGeomTest(unittest.TestCase):
             # Testing equivalence
             self.assertEqual(True, poly == OGRGeometry(p.wkt))
             self.assertEqual(True, poly != prev)
-            
+
             if p.ext_ring_cs:
                 ring = poly[0]
                 self.assertEqual(p.ext_ring_cs, ring.tuple)
                 self.assertEqual(p.ext_ring_cs, poly[0].tuple)
                 self.assertEqual(len(p.ext_ring_cs), ring.point_count)
-            
+
             for r in poly:
                 self.assertEqual('LINEARRING', r.geom_name)
 
@@ -220,10 +246,7 @@ class OGRGeomTest(unittest.TestCase):
 
         # Closing the rings -- doesn't work on GDAL versions 1.4.1 and below:
         # http://trac.osgeo.org/gdal/ticket/1673
-        major, minor1, minor2 = gdal_version().split('.')
-        if major == '1':
-            iminor1 = int(minor1)
-            if iminor1 < 4 or (iminor1 == 4 and minor2.startswith('1')): return
+        if GDAL_VERSION <= (1, 4, 1): return
         poly.close_rings()
         self.assertEqual(10, poly.point_count) # Two closing points should've been added
         self.assertEqual(OGRGeometry('POINT(2.5 2.5)'), poly.centroid)
@@ -231,7 +254,7 @@ class OGRGeomTest(unittest.TestCase):
     def test08_multipolygons(self):
         "Testing MultiPolygon objects."
         prev = OGRGeometry('POINT(0 0)')
-        for mp in multipolygons:
+        for mp in self.geometries.multipolygons:
             mpoly = OGRGeometry(mp.wkt)
             self.assertEqual(6, mpoly.geom_type)
             self.assertEqual('MULTIPOLYGON', mpoly.geom_name)
@@ -246,16 +269,16 @@ class OGRGeomTest(unittest.TestCase):
 
     def test09a_srs(self):
         "Testing OGR Geometries with Spatial Reference objects."
-        for mp in multipolygons:
+        for mp in self.geometries.multipolygons:
             # Creating a geometry w/spatial reference
             sr = SpatialReference('WGS84')
             mpoly = OGRGeometry(mp.wkt, sr)
             self.assertEqual(sr.wkt, mpoly.srs.wkt)
-          
+
             # Ensuring that SRS is propagated to clones.
             klone = mpoly.clone()
             self.assertEqual(sr.wkt, klone.srs.wkt)
-  
+
             # Ensuring all children geometries (polygons and their rings) all
             # return the assigned spatial reference as well.
             for poly in mpoly:
@@ -264,8 +287,8 @@ class OGRGeomTest(unittest.TestCase):
                     self.assertEqual(sr.wkt, ring.srs.wkt)
 
             # Ensuring SRS propagate in topological ops.
-            a, b = topology_geoms[0]
-            a, b = OGRGeometry(a.wkt, sr), OGRGeometry(b.wkt, sr)
+            a = OGRGeometry(self.geometries.topology_geoms[0].wkt_a, sr)
+            b = OGRGeometry(self.geometries.topology_geoms[0].wkt_b, sr)
             diff = a.difference(b)
             union = a.union(b)
             self.assertEqual(sr.wkt, diff.srs.wkt)
@@ -277,7 +300,7 @@ class OGRGeomTest(unittest.TestCase):
             mpoly.srs = SpatialReference(4269)
             self.assertEqual(4269, mpoly.srid)
             self.assertEqual('NAD83', mpoly.srs.name)
-          
+
             # Incrementing through the multipolyogn after the spatial reference
             # has been re-assigned.
             for poly in mpoly:
@@ -319,13 +342,24 @@ class OGRGeomTest(unittest.TestCase):
             self.assertAlmostEqual(trans.x, p.x, prec)
             self.assertAlmostEqual(trans.y, p.y, prec)
 
+    def test09c_transform_dim(self):
+        "Testing coordinate dimension is the same on transformed geometries."
+        ls_orig = OGRGeometry('LINESTRING(-104.609 38.255)', 4326)
+        ls_trans = OGRGeometry('LINESTRING(992385.4472045 481455.4944650)', 2774)
+
+        prec = 3
+        ls_orig.transform(ls_trans.srs)
+        # Making sure the coordinate dimension is still 2D.
+        self.assertEqual(2, ls_orig.coord_dim)
+        self.assertAlmostEqual(ls_trans.x[0], ls_orig.x[0], prec)
+        self.assertAlmostEqual(ls_trans.y[0], ls_orig.y[0], prec)
+
     def test10_difference(self):
         "Testing difference()."
-        for i in xrange(len(topology_geoms)):
-            g_tup = topology_geoms[i]
-            a = OGRGeometry(g_tup[0].wkt)
-            b = OGRGeometry(g_tup[1].wkt)
-            d1 = OGRGeometry(diff_geoms[i].wkt)
+        for i in xrange(len(self.geometries.topology_geoms)):
+            a = OGRGeometry(self.geometries.topology_geoms[i].wkt_a)
+            b = OGRGeometry(self.geometries.topology_geoms[i].wkt_b)
+            d1 = OGRGeometry(self.geometries.diff_geoms[i].wkt)
             d2 = a.difference(b)
             self.assertEqual(d1, d2)
             self.assertEqual(d1, a - b) # __sub__ is difference operator
@@ -334,11 +368,10 @@ class OGRGeomTest(unittest.TestCase):
 
     def test11_intersection(self):
         "Testing intersects() and intersection()."
-        for i in xrange(len(topology_geoms)):
-            g_tup = topology_geoms[i]
-            a = OGRGeometry(g_tup[0].wkt)
-            b = OGRGeometry(g_tup[1].wkt)
-            i1 = OGRGeometry(intersect_geoms[i].wkt)
+        for i in xrange(len(self.geometries.topology_geoms)):
+            a = OGRGeometry(self.geometries.topology_geoms[i].wkt_a)
+            b = OGRGeometry(self.geometries.topology_geoms[i].wkt_b)
+            i1 = OGRGeometry(self.geometries.intersect_geoms[i].wkt)
             self.assertEqual(True, a.intersects(b))
             i2 = a.intersection(b)
             self.assertEqual(i1, i2)
@@ -348,24 +381,22 @@ class OGRGeomTest(unittest.TestCase):
 
     def test12_symdifference(self):
         "Testing sym_difference()."
-        for i in xrange(len(topology_geoms)):
-            g_tup = topology_geoms[i]
-            a = OGRGeometry(g_tup[0].wkt)
-            b = OGRGeometry(g_tup[1].wkt)
-            d1 = OGRGeometry(sdiff_geoms[i].wkt)
+        for i in xrange(len(self.geometries.topology_geoms)):
+            a = OGRGeometry(self.geometries.topology_geoms[i].wkt_a)
+            b = OGRGeometry(self.geometries.topology_geoms[i].wkt_b)
+            d1 = OGRGeometry(self.geometries.sdiff_geoms[i].wkt)
             d2 = a.sym_difference(b)
             self.assertEqual(d1, d2)
             self.assertEqual(d1, a ^ b) # __xor__ is symmetric difference operator
             a ^= b # testing __ixor__
             self.assertEqual(d1, a)
-            
+
     def test13_union(self):
         "Testing union()."
-        for i in xrange(len(topology_geoms)):
-            g_tup = topology_geoms[i]
-            a = OGRGeometry(g_tup[0].wkt)
-            b = OGRGeometry(g_tup[1].wkt)
-            u1 = OGRGeometry(union_geoms[i].wkt)
+        for i in xrange(len(self.geometries.topology_geoms)):
+            a = OGRGeometry(self.geometries.topology_geoms[i].wkt_a)
+            b = OGRGeometry(self.geometries.topology_geoms[i].wkt_b)
+            u1 = OGRGeometry(self.geometries.union_geoms[i].wkt)
             u2 = a.union(b)
             self.assertEqual(u1, u2)
             self.assertEqual(u1, a | b) # __or__ is union operator
@@ -381,7 +412,7 @@ class OGRGeomTest(unittest.TestCase):
 
         # GeometryCollection.add may take an OGRGeometry (if another collection
         # of the same type all child geoms will be added individually) or WKT.
-        for mp in multipolygons:
+        for mp in self.geometries.multipolygons:
             mpoly = OGRGeometry(mp.wkt)
             mp1 = OGRGeometry('MultiPolygon')
             mp2 = OGRGeometry('MultiPolygon')
@@ -399,13 +430,62 @@ class OGRGeomTest(unittest.TestCase):
         mp = OGRGeometry('MULTIPOINT(5 23, 0 0, 10 50)')
         self.assertEqual((0.0, 0.0, 10.0, 50.0), mp.extent)
         # Testing on the 'real world' Polygon.
-        poly = OGRGeometry(polygons[3].wkt)
+        poly = OGRGeometry(self.geometries.polygons[3].wkt)
         ring = poly.shell
         x, y = ring.x, ring.y
         xmin, ymin = min(x), min(y)
         xmax, ymax = max(x), max(y)
         self.assertEqual((xmin, ymin, xmax, ymax), poly.extent)
 
+    def test16_25D(self):
+        "Testing 2.5D geometries."
+        pnt_25d = OGRGeometry('POINT(1 2 3)')
+        self.assertEqual('Point25D', pnt_25d.geom_type.name)
+        self.assertEqual(3.0, pnt_25d.z)
+        self.assertEqual(3, pnt_25d.coord_dim)
+        ls_25d = OGRGeometry('LINESTRING(1 1 1,2 2 2,3 3 3)')
+        self.assertEqual('LineString25D', ls_25d.geom_type.name)
+        self.assertEqual([1.0, 2.0, 3.0], ls_25d.z)
+        self.assertEqual(3, ls_25d.coord_dim)
+
+    def test17_pickle(self):
+        "Testing pickle support."
+        g1 = OGRGeometry('LINESTRING(1 1 1,2 2 2,3 3 3)', 'WGS84')
+        g2 = pickle.loads(pickle.dumps(g1))
+        self.assertEqual(g1, g2)
+        self.assertEqual(4326, g2.srs.srid)
+        self.assertEqual(g1.srs.wkt, g2.srs.wkt)
+
+    def test18_ogrgeometry_transform_workaround(self):
+        "Testing coordinate dimensions on geometries after transformation."
+        # A bug in GDAL versions prior to 1.7 changes the coordinate
+        # dimension of a geometry after it has been transformed.
+        # This test ensures that the bug workarounds employed within
+        # `OGRGeometry.transform` indeed work.
+        wkt_2d = "MULTILINESTRING ((0 0,1 1,2 2))"
+        wkt_3d = "MULTILINESTRING ((0 0 0,1 1 1,2 2 2))"
+        srid = 4326
+
+        # For both the 2D and 3D MultiLineString, ensure _both_ the dimension
+        # of the collection and the component LineString have the expected
+        # coordinate dimension after transform.
+        geom = OGRGeometry(wkt_2d, srid)
+        geom.transform(srid)
+        self.assertEqual(2, geom.coord_dim)
+        self.assertEqual(2, geom[0].coord_dim)
+        self.assertEqual(wkt_2d, geom.wkt)
+
+        geom = OGRGeometry(wkt_3d, srid)
+        geom.transform(srid)
+        self.assertEqual(3, geom.coord_dim)
+        self.assertEqual(3, geom[0].coord_dim)
+        self.assertEqual(wkt_3d, geom.wkt)
+
+    def test19_equivalence_regression(self):
+        "Testing equivalence methods with non-OGRGeometry instances."
+        self.assertNotEqual(None, OGRGeometry('POINT(0 0)'))
+        self.assertEqual(False, OGRGeometry('LINESTRING(0 0, 1 1)') == 3)
+
 def suite():
     s = unittest.TestSuite()
     s.addTest(unittest.makeSuite(OGRGeomTest))
