@@ -6,6 +6,14 @@ from django.core.exceptions import ObjectDoesNotExist
 from django.core.xheaders import populate_xheaders
 from django.db.models.fields import DateTimeField
 from django.http import Http404, HttpResponse
+from django.utils import timezone
+
+import warnings
+warnings.warn(
+    'Function-based generic views have been deprecated; use class-based views instead.',
+    DeprecationWarning
+)
+
 
 def archive_index(request, queryset, date_field, num_latest=15,
         template_name=None, template_loader=loader,
@@ -24,10 +32,10 @@ def archive_index(request, queryset, date_field, num_latest=15,
     if extra_context is None: extra_context = {}
     model = queryset.model
     if not allow_future:
-        queryset = queryset.filter(**{'%s__lte' % date_field: datetime.datetime.now()})
+        queryset = queryset.filter(**{'%s__lte' % date_field: timezone.now()})
     date_list = queryset.dates(date_field, 'year')[::-1]
     if not date_list and not allow_empty:
-        raise Http404, "No %s available" % model._meta.verbose_name
+        raise Http404("No %s available" % model._meta.verbose_name)
 
     if date_list and num_latest:
         latest = queryset.order_by('-'+date_field)[:num_latest]
@@ -67,7 +75,7 @@ def archive_year(request, year, queryset, date_field, template_name=None,
     """
     if extra_context is None: extra_context = {}
     model = queryset.model
-    now = datetime.datetime.now()
+    now = timezone.now()
 
     lookup_kwargs = {'%s__year' % date_field: year}
 
@@ -105,6 +113,8 @@ def archive_month(request, year, month, queryset, date_field,
 
     Templates: ``<app_label>/<model_name>_archive_month.html``
     Context:
+        date_list:
+            List of days in this month with objects
         month:
             (date) this month
         next_month:
@@ -122,7 +132,7 @@ def archive_month(request, year, month, queryset, date_field,
         raise Http404
 
     model = queryset.model
-    now = datetime.datetime.now()
+    now = timezone.now()
 
     # Calculate first and last day of month, for use in a date-range lookup.
     first_day = date.replace(day=1)
@@ -139,6 +149,7 @@ def archive_month(request, year, month, queryset, date_field,
     if last_day >= now.date() and not allow_future:
         lookup_kwargs['%s__lte' % date_field] = now
     object_list = queryset.filter(**lookup_kwargs)
+    date_list = object_list.dates(date_field, 'day')
     if not object_list and not allow_empty:
         raise Http404
 
@@ -160,6 +171,7 @@ def archive_month(request, year, month, queryset, date_field,
         template_name = "%s/%s_archive_month.html" % (model._meta.app_label, model._meta.object_name.lower())
     t = template_loader.get_template(template_name)
     c = RequestContext(request, {
+        'date_list': date_list,
         '%s_list' % template_object_name: object_list,
         'month': date,
         'next_month': next_month,
@@ -194,7 +206,7 @@ def archive_week(request, year, week, queryset, date_field,
         raise Http404
 
     model = queryset.model
-    now = datetime.datetime.now()
+    now = timezone.now()
 
     # Calculate first and last day of week, for use in a date-range lookup.
     first_day = date
@@ -252,7 +264,7 @@ def archive_day(request, year, month, day, queryset, date_field,
         raise Http404
 
     model = queryset.model
-    now = datetime.datetime.now()
+    now = timezone.now()
 
     if isinstance(model._meta.get_field(date_field), DateTimeField):
         lookup_kwargs = {'%s__range' % date_field: (datetime.datetime.combine(date, datetime.time.min), datetime.datetime.combine(date, datetime.time.max))}
@@ -324,7 +336,7 @@ def object_detail(request, year, month, day, queryset, date_field,
         raise Http404
 
     model = queryset.model
-    now = datetime.datetime.now()
+    now = timezone.now()
 
     if isinstance(model._meta.get_field(date_field), DateTimeField):
         lookup_kwargs = {'%s__range' % date_field: (datetime.datetime.combine(date, datetime.time.min), datetime.datetime.combine(date, datetime.time.max))}
@@ -339,11 +351,11 @@ def object_detail(request, year, month, day, queryset, date_field,
     elif slug and slug_field:
         lookup_kwargs['%s__exact' % slug_field] = slug
     else:
-        raise AttributeError, "Generic detail view must be called with either an object_id or a slug/slugfield"
+        raise AttributeError("Generic detail view must be called with either an object_id or a slug/slugfield")
     try:
         obj = queryset.get(**lookup_kwargs)
     except ObjectDoesNotExist:
-        raise Http404, "No %s found for" % model._meta.verbose_name
+        raise Http404("No %s found for" % model._meta.verbose_name)
     if not template_name:
         template_name = "%s/%s_detail.html" % (model._meta.app_label, model._meta.object_name.lower())
     if template_name_field:
