@@ -1,9 +1,5 @@
 from django.core.management.base import LabelCommand
-# Python 2.3 doesn't have sorted()
-try:
-    sorted
-except NameError:
-    from django.utils.itercompat import sorted
+
 
 class Command(LabelCommand):
     help = "Test Label-based commands"
